package com.myco.jmstest;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;

import javax.jms.Queue;

import org.apache.log4j.Logger;
import org.springframework.jms.JmsException;
import org.springframework.jms.core.JmsTemplate;

public class JmsSender {

    private static Logger LOGGER = Logger.getLogger(JmsSender.class);

    private final JmsTemplate template;
    private final Queue queue;
    
    private long lastPrintedTime = 0;
    
    private AtomicBoolean currentlyPrinting = new AtomicBoolean(false);
    private AtomicLong messagesSent = new AtomicLong();
    private AtomicLong totalSent = new AtomicLong();
    private boolean isShutdown = false;
    
    private final ExecutorService executorService;
    private final LinkedBlockingQueue<String> sendQueue;

    
    public JmsSender(JmsTemplate template, Queue queue) {
        this.template = template;
        this.queue = queue;
        
        sendQueue = new LinkedBlockingQueue<String>(10000);
        
        executorService = Executors.newFixedThreadPool(25, new ThreadFactory() {
            private int threadCount = 0;
            
            @Override
            public Thread newThread(Runnable r) {
                return new Thread(r, "JmsSender-" + threadCount++);
            }
        });
        
        for(int i = 0; i < 25; i++) {
            executorService.execute(new JmsSenderWorker());
        }
    }
    
    private class JmsSenderWorker implements Runnable {
        @Override
        public void run() {
            while(true) {
                
                if(isShutdown) {
                    break;
                }
                
                try {
                    String msg = sendQueue.take();
                    if(msg != null) {
                        try {
                            template.convertAndSend(queue, msg);
                            messagesSent.incrementAndGet();
                            totalSent.incrementAndGet();
                            
                            long currentTime = System.currentTimeMillis();
                            if(currentTime - lastPrintedTime > 5000) {
                                if(currentlyPrinting.compareAndSet(false, true)) {
                                    if(currentTime - lastPrintedTime > 5000) {
                                        long numSent = messagesSent.getAndSet(0);
                                        LOGGER.info("Messages sent: " + numSent);
                                        lastPrintedTime = currentTime;
                                    }
                                    currentlyPrinting.set(false);
                                }
                            }
                        } catch (JmsException e) {
                            e.printStackTrace();
                        }
                    }
                } catch(InterruptedException e) {
                    Thread.currentThread().interrupt();
                    break;
                }
            }
        }
    }
    
    public void sendMessage(String msg) throws InterruptedException {
        while(true) {
            if(sendQueue.offer(msg)) {
                break;
            }
            Thread.sleep(1);
        }
    }
    
    public long getTotalSent() {
        return totalSent.get();
    }
    
    public void shutdown() throws InterruptedException {
        isShutdown = true;
        executorService.shutdown();
        if(!executorService.awaitTermination(5, TimeUnit.SECONDS)) {
            executorService.shutdownNow();
            if(!executorService.awaitTermination(5, TimeUnit.SECONDS)) {
                LOGGER.info("Could not shutdown executor service");
            }
        }
    }
}
